@@ -48,9 +48,7 @@
 #include <QtCore/QList>
 #include <QtCore/QDir>
 #include <QtCore/QSettings>
-#if (QT_VERSION >= QT_VERSION_CHECK(5,14,0))
 #include <QtCore/QStringView>
-#endif
 #include <QtGui/QPainter>
 #include <QImageReader>
 #include <QXmlStreamReader>
@@ -111,11 +109,7 @@ class QIconCacheGtkReader
 {
 public:
     explicit QIconCacheGtkReader(const QString &themeDir);
-#if (QT_VERSION >= QT_VERSION_CHECK(5,14,0))
     QVector<const char *> lookup(QStringView);
-#else
-    QVector<const char *> lookup(const QStringRef &);
-#endif
     bool isValid() const { return m_isValid; }
     bool reValid(bool infoRefresh);
 private:
@@ -226,11 +220,7 @@ static quint32 icon_name_hash(const char *p)
     For example, this would return { "32x32/apps", "24x24/apps" , ... }
  */
 
-#if (QT_VERSION >= QT_VERSION_CHECK(5,14,0))
 QVector<const char *> QIconCacheGtkReader::lookup(QStringView name)
-#else
-QVector<const char *> QIconCacheGtkReader::lookup(const QStringRef &name)
-#endif
 {
     QVector<const char *> ret;
     if (!isValid() || name.isEmpty())
@@ -350,7 +340,6 @@ XdgIconTheme::XdgIconTheme(const QString &themeName)
         m_parents = indexReader.value(
                 QLatin1String("Icon Theme/Inherits")).toStringList();
         m_parents.removeAll(QString());
-        m_parents.removeAll(QLatin1String("hicolor"));
 
         // Ensure a default platform fallback for all themes
         if (m_parents.isEmpty()) {
@@ -412,11 +401,7 @@ QThemeIconInfo XdgIconLoader::findIconHelper(const QString &themeName,
     const QString xpmext(QLatin1String(".xpm"));
 
 
-#if (QT_VERSION >= QT_VERSION_CHECK(5,14,0))
     QStringView iconNameFallback(iconName);
-#else
-    QStringRef iconNameFallback(&iconName);
-#endif
 
     // Iterate through all icon's fallbacks in current theme
     if (info.entries.isEmpty()) {
@@ -499,6 +484,38 @@ QThemeIconInfo XdgIconLoader::findIconHelper(const QString &themeName,
             if (!info.entries.isEmpty()) // success
                 break;
         }
+
+        // make sure that hicolor is also searched before dash fallbacks
+        if (info.entries.isEmpty()
+            && !parents.contains(QLatin1String("hicolor"))
+            && !visited.contains(QLatin1String("hicolor"))) {
+            info = findIconHelper(QLatin1String("hicolor"), iconName, visited);
+        }
+    }
+
+    if (info.entries.isEmpty()) {
+        // Also, consider Qt's fallback search paths (which are not defined by Freedesktop)
+        // if the icon is not found in any inherited theme
+        const auto fallbackPaths = QIcon::fallbackSearchPaths();
+        for (const auto &fallbackPath : fallbackPaths) {
+            const QString pngPath = fallbackPath + QLatin1Char('/') + iconName + pngext;
+            if (QFile::exists(pngPath)) {
+                PixmapEntry *iconEntry = new PixmapEntry;
+                QIconDirInfo dirInfo(fallbackPath);
+                iconEntry->dir = dirInfo;
+                iconEntry->filename = pngPath;
+                info.entries.prepend(iconEntry);
+            } else {
+                const QString svgPath = fallbackPath + QLatin1Char('/') + iconName + svgext;
+                if (gSupportsSvg && QFile::exists(svgPath)) {
+                    ScalableEntry *iconEntry = new ScalableEntry;
+                    QIconDirInfo dirInfo(fallbackPath);
+                    iconEntry->dir = dirInfo;
+                    iconEntry->filename = svgPath;
+                    info.entries.append(iconEntry);
+                }
+            }
+        }
     }
 
     if (dashFallback && info.entries.isEmpty()) {
@@ -554,23 +571,18 @@ QThemeIconInfo XdgIconLoader::loadIcon(const QString &name) const
         QStringList visited;
         auto info = findIconHelper(theme_name, name, visited, true);
         if (info.entries.isEmpty()) {
-            const auto hicolorInfo = findIconHelper(QLatin1String("hicolor"), name, visited, true);
-            if (hicolorInfo.entries.isEmpty()) {
-                const auto unthemedInfo = unthemedFallback(name, QIcon::themeSearchPaths());
-                if (unthemedInfo.entries.isEmpty()) {
-                    /* Freedesktop standard says to look in /usr/share/pixmaps last */
-                    const QStringList pixmapPath = (QStringList() << QString::fromLatin1("/usr/share/pixmaps"));
-                    const auto pixmapInfo = unthemedFallback(name, pixmapPath);
-                    if (pixmapInfo.entries.isEmpty()) {
-                        return QThemeIconInfo();
-                    } else {
-                        return pixmapInfo;
-                    }
+            const auto unthemedInfo = unthemedFallback(name, QIcon::themeSearchPaths());
+            if (unthemedInfo.entries.isEmpty()) {
+                /* Freedesktop standard says to look in /usr/share/pixmaps last */
+                const QStringList pixmapPath = (QStringList() << QString::fromLatin1("/usr/share/pixmaps"));
+                const auto pixmapInfo = unthemedFallback(name, pixmapPath);
+                if (pixmapInfo.entries.isEmpty()) {
+                    return QThemeIconInfo();
                 } else {
-                    return unthemedInfo;
+                    return pixmapInfo;
                 }
             } else {
-                return hicolorInfo;
+                return unthemedInfo;
             }
         } else {
             return info;
